@@ -7,12 +7,16 @@ meanvarESNuni = function(a=-Inf,b=Inf,mu,Sigma,lambda,tau){
       mub    = lambda*tau*Gamma/s
       return(meanvarNuni(a,b,mu-mub,Gamma))
     }
+    F0     = AcumESN(b,mu,Sigma,lambda,tau) - AcumESN(a,mu,Sigma,lambda,tau)
+    if(F0 < 1e-250){
+      val = select(a,b,mu,s)
+      return(list(mean = val,EYY = 0.0001 + val^2, varcov = 0.0001))
+    }
     phi    = lambda/sqrt(1+sum(lambda^2))
     eta    = invmills(tau,0,sqrt(1+lambda^2))
     Gamma  = Sigma/(1+lambda^2)
     mub    = lambda*tau*Gamma/s
-    F0N    = pnorm(b,mu-mub,sqrt(Gamma))-pnorm(a,mu-mub,sqrt(Gamma))
-    F0     = AcumESN(b,mu,Sigma,lambda,tau) - AcumESN(a,mu,Sigma,lambda,tau)
+    F0N    = pnorm2(a,b,mu-mub,sqrt(Gamma))
     da     = dmvESN1(a,mu,Sigma,lambda,tau)
     db     = dmvESN1(b,mu,Sigma,lambda,tau)
     F1     = mu*F0 + Sigma*(da - db) + lambda*s*eta*F0N
